import os
import json
import logging.config


def setup_logging(
        default_level=logging.INFO,
        default_path="{}/logging.json".format(
            os.getenv(
                "LOG_DIR",
                os.path.dirname(os.path.realpath(__file__)))),
        env_key="LOG_CFG",
        config_name=None):
    """setup_logging

    Setup logging configuration

    :param default_level: level to log
    :param default_path: path to config (optional)
    :param env_key: path to config in this env var
    :param config_name: filename for config

    """
    path = default_path
    file_name = default_path.split("/")[-1]
    if config_name:
        file_name = config_name

    path = "{}/{}".format(
                "/".join(default_path.split("/")[:-1]),
                file_name)

    value = os.getenv(env_key, None)
    if value:
        path = value
    if os.path.exists(path):
        with open(path, "rt") as f:
            config = json.load(f)
        logging.config.dictConfig(config)
        return

    else:
        cwd_path = os.getcwd() + "/network_pipeline/log/{}".format(
                                    file_name)
        if os.path.exists(cwd_path):
            with open(cwd_path, "rt") as f:
                config = json.load(f)
            logging.config.dictConfig(config)
            return
        rels_path = os.getcwd() + "/../log/{}".format(
                                    file_name)
        if os.path.exists(rels_path):
            with open(rels_path, "rt") as f:
                config = json.load(f)
            logging.config.dictConfig(config)
            return
        else:
            logging.basicConfig(level=default_level)
            return
# end of setup_logging


def build_logger(
    name=os.getenv(
        "LOG_NAME",
        "netp"),
    config="logging.json",
    log_level=logging.INFO,
    log_config_path="{}/logging.json".format(
        os.getenv(
            "LOG_CFG",
            os.path.dirname(os.path.realpath(__file__))))):
    """build_logger

    :param name: name that shows in the logger
    :param config: name of the config file
    :param log_level: level to log
    :param log_config_path: path to log config file
    """
    use_config = ("./log/{}").format(
                    "{}".format(
                        config))
    if not os.path.exists(use_config):
        use_config = log_config_path
        if not os.path.exists(use_config):
            use_config = ("./network_pipeline/log/{}").format(
                            "logging.json")
    # find the log processing

    setup_logging(
        default_level=log_level,
        default_path=use_config)

    return logging.getLogger(name)
# end of build_logger


def build_colorized_logger(
    name=os.getenv(
        "LOG_NAME",
        "netp"),
    config="colors-logging.json",
    log_level=logging.INFO,
    log_config_path="{}/logging.json".format(
        os.getenv(
            "LOG_CFG",
            os.path.dirname(os.path.realpath(__file__))))):
    """build_colorized_logger

    :param name: name that shows in the logger
    :param config: name of the config file
    :param log_level: level to log
    :param log_config_path: path to log config file
    """

    override_config = os.getenv(
        "SHARED_LOG_CFG",
        None)
    debug_log_config = bool(os.getenv(
        "DEBUG_SHARED_LOG_CFG",
        "0") == "1")
    if override_config:
        if debug_log_config:
            print((
                "creating logger config env var: "
                "SHARED_LOG_CFG={}".format(
                    override_config)))
        if os.path.exists(override_config):
            setup_logging(
                default_level=log_level,
                default_path=override_config)
            return logging.getLogger(name)
        if debug_log_config:
            print((
                "Failed to find log config using env var: "
                "SHARED_LOG_CFG={}".format(
                    override_config)))
    else:
        if debug_log_config:
            print((
                "Not using shared logging env var: "
                "SHARED_LOG_CFG={}".format(
                    override_config)))
    # allow a shared log config across all components

    use_config = ("{}").format(
        config)

    if not os.path.exists(use_config):
        use_config = ("./network_pipeline/log/{}").format(
                            config)
        if not os.path.exists(use_config):
            use_config = log_config_path
            if not os.path.exists(use_config):
                use_config = ("./log/{}").format(
                            config)
                if not os.path.exists(use_config):
                    use_config = ("./network_pipeline/log/{}").format(
                                "logging.json")
                # find the last log config backup from the base of the repo
            # find the log config from the defaults with the env LOG_CFG
        # find the log config from the base of the repo
    # find the log config by the given path

    setup_logging(
        default_level=log_level,
        default_path=use_config)

    return logging.getLogger(name)
# end of build_colorized_logger



import time


class SimulatedWorkTask(object):

    def __init__(self, a, b):
        self.a = a
        self.b = b

    def __call__(self):
        time.sleep(0.1)  # pretend to take some time to do the work
        return "{} * {} = {}".format(self.a, self.b, self.a * self.b)

    def __str__(self):
        return str(("{} * {} = {}")
                   .format(self.a,
                           self.b,
                           float(self.a * self.b)))
# end of SimulatedWorkTask



#!/usr/bin/env python

import logging
import scapy.all as scapy
from celery_connectors.utils import ev
from network_pipeline.log.setup_logging import setup_logging
from network_pipeline.handle_packets import handle_packets


setup_logging()
# scapy capture agent
name = "cap"
log = logging.getLogger(name)


def example_capture():
    """example_capture

    An example capture script

    Change the network interface by ``export CAP_DEVICE=eth0``

    """

    dev = ev(
        "CAP_DEVICE",
        "lo")

    """
    Ignore ports for forwarding to consolidators:

    Redis Internal VM: 6379, 16379
    RabbitMQ Internal VM: 5672, 15672, 25672
    """

    # http://biot.com/capstats/bpf.html
    custom_filter = ("(udp and portrange 10000-17001) "
                     "or (tcp and portrange 80) "
                     "or arp "
                     "or icmp")

    log.info(("starting device={} filter={}")
             .format(
                dev,
                custom_filter))

    scapy.sniff(
        filter=custom_filter,
        prn=handle_packets)

    log.info("done")

# end of example_capture


if __name__ == "__main__":
    example_capture()



#!/usr/bin/env python

import os
import socket
import uuid
import datetime


def send_udp_message():
    """send_udp_message

    Send a ``UDP`` message to port 80 by default.

    Environment variables:

    ``UDP_SEND_TO_HOST`` - host ip address
    ``UDP_SEND_TO_PORT`` - send to this UDP port

    """
    host = os.getenv(
        "UDP_SEND_TO_HOST",
        "0.0.0.0").strip().lstrip()
    port = int(os.getenv(
        "UDP_SEND_TO_PORT",
        "17000").strip().lstrip())

    need_response = os.getenv("NEED_RESPONSE", "0") == "1"

    msg = os.getenv(
            "MSG",
            "testing UDP msg time={} - {}".format(
                datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                uuid.uuid4()))

    server_address = (host, port)

    client = socket.socket(socket.AF_INET,
                           socket.SOCK_DGRAM)

    print(("sending UDP: "
           "address={} msg={}")
          .format(server_address,
                  msg))
    client.sendto(msg.encode(), server_address)
    if need_response:
        data = client.recv(1024).decode()
        print(data)
    client.close()
# end of send_udp_message


if __name__ == '__main__':
    send_udp_message()



#!/usr/bin/env python

import os
import sys
import logging
from celery_connectors.utils import ev
from network_pipeline.log.setup_logging import setup_logging
from network_pipeline.consts import VALID
from network_pipeline.build_training_request import build_training_request
from keras.models import Sequential
from keras.layers import Dense


setup_logging(config_name="modelers.json")
name = "keras-dnn"
log = logging.getLogger(name)
log.info("start - {}".format(name))


def build_new_deep_neural_network_from_env_variables():
    """build_new_deep_neural_network_from_env_variables

    Build a new deep neural network from environment variables:

    ``CSV_FILE`` - file to process created during prepare dataset
    ``CSV_META_FILE`` - metadata header file created during prepare dataset
    ``PREDICT_FEATURE`` - column to predict
    ``TEST_SIZE`` - split data into percentage of test to training
    """

    csv_file = ev(
        "CSV_FILE",
        "/tmp/cleaned_attack_scans.csv")
    meta_file = ev(
        "CSV_META_FILE",
        "/tmp/cleaned_metadata.json")
    predict_feature = ev(
        "PREDICT_FEATURE",
        "label_value")
    test_size = float(ev(
        "TEST_SIZE",
        "0.20"))

    if not os.path.exists(csv_file):
        log.error(("missing csv_file={}")
                  .format(csv_file))
        sys.exit(1)

    res = build_training_request(
            csv_file=csv_file,
            meta_file=meta_file,
            predict_feature=predict_feature,
            test_size=test_size)

    if res["status"] != VALID:
        log.error(("Stopping for status={} "
                   "errors: {}")
                  .format(
                    res["status"],
                    res["err"]))
        sys.exit(1)
    else:
        log.info(("built_training_request={} "
                  "features={} ignore={}")
                 .format(
                    res["status"],
                    res["features_to_process"],
                    res["ignore_features"]))
    # end of validating the training request

    log.info("ready for training")

    log.info("creating Keras - sequential model")

    # create the model
    model = Sequential()
    model.add(Dense(8,
                    input_dim=len(res["features_to_process"]),
                    kernel_initializer="uniform",
                    activation="relu"))
    model.add(Dense(6,
                    kernel_initializer="uniform",
                    activation="relu"))
    model.add(Dense(1,
                    kernel_initializer="uniform",
                    activation="sigmoid"))

    log.info("compiling model")

    # compile the model
    model.compile(
        loss="binary_crossentropy",
        optimizer="adam",
        metrics=["accuracy"])

    log.info("fitting model - please wait")

    # fit the model
    model.fit(
        res["X_train"],
        res["Y_train"],
        validation_data=(res["X_test"],
                         res["Y_test"]),
        epochs=50,
        batch_size=2,
        verbose=1)

    # evaluate the model
    scores = model.evaluate(
        res["X_test"],
        res["Y_test"])

    log.info(("Accuracy: {}")
             .format(
                scores[1] * 100))

# end of build_new_deep_neural_network_from_env_variables


if __name__ == "__main__":
    build_new_deep_neural_network_from_env_variables()



import logging
import multiprocessing
from network_pipeline.log.setup_logging import setup_logging

setup_logging()
name = "pcktr"
log = logging.getLogger(name)


class WorkerToProcessPackets(multiprocessing.Process):

    def __init__(self,
                 name,
                 task_queue,
                 result_queue,
                 shutdown_msg="SHUTDOWN",
                 need_response=False,
                 callback=None):
        """__init__

        :param name: name of the consumer
        :param task_queue: queue for tasks to process
        :param result_queue: send results to back here
        :param shutdown_msg: custom poison pill  shutdown msg
        :param need_response: send a response back on result_queue
        :param callback: method for processing packets
        """
        multiprocessing.Process.__init__(self)
        self.name = name
        self.task_queue = task_queue
        self.need_response = need_response
        self.result_queue = result_queue

        self.callback = callback
        self.shutdown_msg = shutdown_msg
    # end of __init__

    def run(self):
        """run"""
        if self.callback:
            log.info(("{} - using callback={}")
                     .format(self.name,
                             self.callback))
            self.callback(name=name,
                          task_queue=self.task_queue,
                          result_queue=self.result_queue,
                          shutdown_msg=self.shutdown_msg)
        else:

            log.info("did not find a callback method "
                     "using - using default handler")

            proc_name = self.name
            while True:
                next_task = self.task_queue.get()
                if next_task:
                    if str(next_task) == self.shutdown_msg:
                        # Poison pill means shutdown
                        log.info(("{}: Exiting msg={}")
                                 .format(self.name,
                                         next_task))
                        self.task_queue.task_done()
                        break
                log.info(("Consumer: {} {}")
                         .format(proc_name, next_task))
                self.task_queue.task_done()

                if self.need_response:
                    answer = "processed: {}".format(next_task())
                    self.result_queue.put(answer)
        # end of if custome callback handler or not

        return
    # end of run

# end of WorkerToProcessPackets



import json
import datetime


def rnow(f="%Y-%m-%d %H:%M:%S"):
    """rnow

    :param f: format for the string
    """
    return datetime.datetime.now().strftime(f)
# end of rnow


def ppj(json_data):
    """ppj

    :param json_data: dictionary to print
    """
    return str(json.dumps(json_data,
                          sort_keys=True,
                          indent=4,
                          separators=(',', ': ')))
# end of ppj



#!/usr/bin/env python

import os
import sys
import logging
import socket
import datetime
import time
import json
from network_pipeline.consts import INCLUDED_IGNORE_KEY
from network_pipeline.consts import VALID
from network_pipeline.consts import TCP
from network_pipeline.consts import UDP
from network_pipeline.consts import ICMP
from network_pipeline.consts import ARP
from network_pipeline.log.setup_logging import setup_logging
from network_pipeline.utils import ppj

# from celery_connectors.publisher import Publisher

setup_logging()

# consolidator - receives packets from network agents
name = "cdr"
log = logging.getLogger(name)


def handle_processing_packets():
    """handle_processing_packets

    Replacement packet processing engine. This is not done.

    """

    host = os.getenv(
        "LISTEN_ON_HOST",
        "127.0.0.1").strip().lstrip()
    port = int(os.getenv(
        "LISTEN_ON_PORT",
        "80").strip().lstrip())
    backlog = int(os.getenv(
        "LISTEN_BACKLOG",
        "5").strip().lstrip())
    size = int(os.getenv(
        "LISTEN_SIZE",
        "102400").strip().lstrip())
    sleep_in_seconds = float(os.getenv(
        "LISTEN_SLEEP",
        "0.5").strip().lstrip())
    needs_response = bool(os.getenv(
        "LISTEN_SEND_RESPONSE",
        "0").strip().lstrip() == "1")
    shutdown_hook = os.getenv(
        "LISTEN_SHUTDOWN_HOOK",
        "/tmp/shutdown-listen-server-{}-{}".format(
            host,
            port)).strip().lstrip()
    filter_key = os.getenv(
        "IGNORE_KEY",
        INCLUDED_IGNORE_KEY).strip().lstrip()

    if os.path.exists(shutdown_hook):
        log.info(("Please remove the shutdown hook file: "
                  "\nrm -f {}")
                 .format(
                    shutdown_hook))
        sys.exit(1)

    default_filter_key = filter_key
    bytes_for_filter_key = len(default_filter_key)
    offset_to_filter_key = (-1 * bytes_for_filter_key)
    offset_to_msg = offset_to_filter_key - 1

    now = datetime.datetime.now().isoformat()
    log.info(("{} - Starting Server address={}:{} "
              "backlog={} size={} sleep={} shutdown={} "
              "filter_key={}")
             .format(
                now,
                host,
                port,
                backlog,
                size,
                sleep_in_seconds,
                shutdown_hook,
                default_filter_key))

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.bind((host, port))
    s.listen(backlog)
    client, address = s.accept()

    midx = 0
    while 1:
        data = None
        address = None
        ignore_key = None

        try:
            if not client:
                client, address = s.accept()
        except Exception as e:
            log.error(("socket accept with ex={}")
                      .format(
                        e))
        try:
            if client:
                data = client.recv(size)
        except Exception as e:
            log.error(("recv - disconnected with ex={}")
                      .format(
                        e))
        if data:
            now = datetime.datetime.now().isoformat()
            packet_to_process = data[0:offset_to_msg]
            ignore_key = data[offset_to_filter_key:]
            log.info(("decoding data={} key={}")
                     .format(
                        packet_to_process,
                        ignore_key))

            msg = None
            try:
                msg = json.loads(
                            packet_to_process.decode("utf-8"))
            except Exception as e:
                msg = None
                log.error(("Invalid data={} with ex={}")
                          .format(
                            packet_to_process,
                            e))

            if msg:

                log.info(("received msg={} "
                          "data={} replying - ignore='{}'")
                         .format(
                            ppj(msg),
                            packet_to_process,
                            ignore_key))

                if msg["status"] == VALID:
                    if msg["data_type"] == TCP:
                        log.info("TCP")
                    elif msg["data_type"] == UDP:
                        log.info("TCP")
                    elif msg["data_type"] == ARP:
                        log.info("TCP")
                    elif msg["data_type"] == ICMP:
                        log.info("TCP")
                    else:
                        log.error(("unsuppported type={}")
                                  .format(
                                    msg["data_type"]))
                    # end of supported eth protocol message types
                else:
                    log.error(("unsuppported msg status={}")
                              .format(
                                msg["status"]))
                # end if msg was VALID
            # end of if found msg

            midx += 1
            if midx > 1000000:
                midx = 0
        else:
            log.debug("ignoring invalid data")
        # end of if valid msg or not

        if needs_response:
            client.send(ignore_key)
        else:
            log.info("no response")
            time.sleep(sleep_in_seconds)

        if os.path.exists(shutdown_hook):
            now = datetime.datetime.now().isoformat()
            log.info(("{} detected shutdown "
                      "file={}")
                     .format(
                        now,
                        shutdown_hook))
    # end of loop

    log.info("shutting down")
    client.close()

    log.info("done")

# end of handle_processing_packets


if __name__ == "__main__":
    handle_processing_packets()



#!/usr/bin/env python

from celery_connectors.kombu_subscriber import KombuSubscriber
from network_pipeline.consts import FORWARD_BROKER_URL
from network_pipeline.consts import FORWARD_SSL_OPTIONS
from network_pipeline.consts import FORWARD_QUEUE
from spylunking.log.setup_logging import build_colorized_logger
from network_pipeline.record_packets_to_csv import RecordPacketsToCSV


name = "kombu-redis-sub"
log = build_colorized_logger(
    name=name,
    config="packets-redis-logging.json")

log.info("start - {}".format(name))

agg = RecordPacketsToCSV()


def recv_msg(body,
             message):
    """recv_msg

    Handler method - fires when a messages is consumed from
    the ``FORWARD_QUEUE`` queue running in the ``FORWARD_BROKER_URL``
    broker.

    :param body: message body
    :param message: message object can ack, requeue or reject
    """

    log.info(("callback received msg "))

    agg.handle_msg(
        body=body,
        org_message=message)
# end of recv_msg


def consume_network_packet_messages_from_redis():
    """consume_network_packet_messages_from_redis

    Setup a ``celery_connectors.KombuSubscriber`` to consume meessages
    from the ``FORWARD_BROKER_URL`` broker in the ``FORWARD_QUEUE``
    queue.
    """
    # end of recv_message
    # Initialize KombuSubscriber
    sub = KombuSubscriber(
        name,
        FORWARD_BROKER_URL,
        FORWARD_SSL_OPTIONS)

    # Now consume:
    seconds_to_consume = 10.0
    heartbeat = 60
    serializer = "application/json"
    queue = FORWARD_QUEUE

    sub.consume(
        callback=recv_msg,
        queue=queue,
        exchange=None,
        routing_key=None,
        serializer=serializer,
        heartbeat=heartbeat,
        time_to_wait=seconds_to_consume)

    log.info("end - {}".format(name))
# end of consume_network_packet_messages_from_redis


if __name__ == "__main__":
    consume_network_packet_messages_from_redis()



#!/usr/bin/env python

import scapy.all as scapy
from celery_connectors.utils import ev
from spylunking.log.setup_logging import build_colorized_logger
from network_pipeline.handle_packets import handle_packets


name = "cap-ssh"
log = build_colorized_logger(
    name=name,
    config="capture-ssh-logging.json")


def capture_tcp_packets_over_ssh():
    """capture_tcp_packets_over_ssh

    Capture ``TCP`` packets over ssh
    and call the ``handle_packets`` method

    Change the network interface by ``export CAP_DEVICE=eth0``

    """
    dev = ev(
        "CAP_DEVICE",
        "lo")

    """
    Ignore ports for forwarding to consolidators:

    Redis VM: 6379, 16379
    RabbitMQ VM: 5672, 15672, 25672

    """

    # http://biot.com/capstats/bpf.html
    default_filter = ("tcp and ( port 22 )")
    custom_filter = ev(
        "NETWORK_FILTER",
        default_filter)

    log.info(("starting device={} filter={}")
             .format(
                dev,
                custom_filter))

    scapy.sniff(
        filter=custom_filter,
        prn=handle_packets)

    log.info("done")

# end of capture_tcp_packets_over_ssh


if __name__ == "__main__":
    capture_tcp_packets_over_ssh()



import os
import sys
import warnings
import unittest

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

try:
    from distutils.command.build_py import build_py_2to3 as build_py
except ImportError:
    from distutils.command.build_py import build_py


cur_path, cur_script = os.path.split(sys.argv[0])
os.chdir(os.path.abspath(cur_path))

install_requires = [
    "antinex-client",
    "ansible>=1.9",
    "celery-connectors",
    "coverage",
    "colorlog",
    "flake8<=3.4.1",
    "future",
    "netifaces",
    "pandas",
    "pep8>=1.7.1",
    "pipenv",
    "pydocstyle",
    "pycodestyle<=2.3.1",
    "pylint",
    "python-logstash",
    "python-owasp-zap-v2.4",
    "python-dateutil<2.7.0",
    "scapy-python3",
    "spylunking",
    "tox",
    "unittest2",
    "mock"
]


if sys.version_info < (3, 5):
    warnings.warn(
        "Less than Python 3.5 is not supported.",
        DeprecationWarning)


def networkpipeline_test_suite():
    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover("tests", pattern="test_*.py")
    return test_suite


# Do not import networkpipeline module here, since deps may not be installed
sys.path.insert(0, os.path.join(os.path.dirname(__file__), "network_pipeline"))

setup(
    name="network-pipeline",
    cmdclass={"build_py": build_py},
    version="1.0.33",
    description="Distributed Network Packet Analysis Pipeline " +
    "for Layer 2, 3 and 4 Frames",
    long_description="" +
    "Python 3 AI-ready framework for recording network traffic in "
    "a data pipeline. Once recorded, you can train a " +
    "deep neural network (DNN) " +
    "to identify attack and non-attack traffic on your network. " +
    "Included demo DNN has over 83% accuracy predicting " +
    "attack vs non-attack records. " +
    "" +
    "Currently supports recording ethernet and arp (layer 2), " +
    "ipv4, ipv6 and icmp (layer 3) and also " +
    "tcp, udp frames (layer 4) frames and datagrams. " +
    "Messages are auto-forwarded over to redis or rabbitmq " +
    "for distributed processing in realtime. " +
    "\n" +
    "Why should I use this? " +
    "This framework can help " +
    "build, train and tune your own " +
    "defensive machine learning models to help defend your " +
    "own infrastructure at the network layer. Once the data " +
    "is auto-saved as a csv file, then you can build models " +
    "within Jupyter notebooks: " +
    "https://github.com/jay-johnson/celery-connectors" +
    "#running-jupyterhub-with-postgres-and-ssl " +
    "or your ML/AI framework of choice. " +
    "\n" +
    "This pip also has an example for training a Keras Deep "
    "Neural Network model to predict attack and non-attack records " +
    "using a captured and prepared dataset. " +
    "\n" +
    "There are test tools installed with this pip to quickly " +
    "send mock: TCP, UDP, ARP and ICMP packets. " +
    "\n" +
    "This build currently utilizes scapy-python3 " +
    "for packet recording: " +
    "https://github.com/phaethon/scapy " +
    "\n" +
    "Future builds will utilize the multiprocessing engine " +
    "included but does not filter src/dst ports correctly yet." +
    "The license will be full Apache 2 once that migration " +
    "is done." +
    "",
    author="Jay Johnson",
    author_email="jay.p.h.johnson@gmail.com",
    url="https://github.com/jay-johnson/network-pipeline",
    packages=[
        "network_pipeline",
        "network_pipeline.scripts",
        "network_pipeline.scripts.builders",
        "network_pipeline.scripts.modelers",
        "network_pipeline.scripts.tools",
        "network_pipeline.log"
    ],
    package_data={},
    install_requires=install_requires,
    test_suite="setup.networkpipeline_test_suite",
    tests_require=[
    ],
    scripts=[
        "network_pipeline/scripts/arp_send_msg.py",
        "network_pipeline/scripts/capture_arp.py",
        "network_pipeline/scripts/capture_icmp.py",
        "network_pipeline/scripts/capture_ssh.py",
        "network_pipeline/scripts/capture_tcp.py",
        "network_pipeline/scripts/capture_telnet.py",
        "network_pipeline/scripts/capture_udp.py",
        "network_pipeline/scripts/icmp_send_msg.py",
        "network_pipeline/scripts/listen_udp_port.py",
        "network_pipeline/scripts/listen_tcp_port.py",
        "network_pipeline/scripts/network_agent.py",
        "network_pipeline/scripts/tcp_send_msg.py",
        "network_pipeline/scripts/udp_send_msg.py",
        "network_pipeline/scripts/packets_redis.py",
        "network_pipeline/scripts/packets_rabbitmq.py",
        "network_pipeline/scripts/builders/prepare_dataset.py",
        "network_pipeline/scripts/modelers/keras_dnn.py",
        "network_pipeline/scripts/start-container.sh"
    ],
    use_2to3=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ])



#!/usr/bin/env python

import os
import socket
import struct
import netifaces
from binascii import unhexlify

interface_name = os.getenv(
    "ARP_INTERFACE",
    "lo").strip().lstrip()
network_details = netifaces.ifaddresses(
    interface_name)
ipaddress = network_details[2][0]['addr']
macaddress = network_details[17][0]['addr']


class Ethernet(object):

    """
    Generic Ethernet Frame class
    """

    def __init__(self):
        self.dst = None
        self.src = None
        self.etype = None


class Arp(object):

    """
    Generic ARP Frame class
    """

    def __init__(self):
        self.htype = None
        self.ptype = None
        self.hsize = None
        self.psize = None
        self.op = None
        self.shwa = None
        self.spa = None
        self.thwa = None
        self.tpa = None
        self.padd = None


host = os.getenv(
            "UDP_LISTEN_ON_HOST",
            "127.0.0.1").strip().lstrip()
port = int(os.getenv(
            "UDP_LISTEN_ON_PORT",
            "17000").strip().lstrip())

need_response = os.getenv("NEED_RESPONSE", "0") == "1"

if __name__ == "__main__":
    server_address = (host, port)

    eth = Ethernet()
    eth.dst = unhexlify("ffffffffffff")
    eth.src = unhexlify("e09d312efe3c")  # your NIC MAC
    eth.etype = 806

    arp = Arp()
    arp.htype = 0x01  # 2 bytes
    arp.ptype = 0x0800  # 2 bytes
    arp.hsize = 0x06  # 1 byte
    arp.psize = 0x04  # 1 byte
    arp.op = 0x01  # 2 bytes
    arp.shwa = unhexlify("e09d312efe3c")   # your NIC MAC
    arp.spa = socket.inet_aton("192.168.0.56")   # your host IP
    arp.thwa = unhexlify("000000000000")
    arp.padd = unhexlify("49206c6f7665206e6574776f726b696e6721")

    # should be any IP from your network if
    # hen using the same IP as you host you will send GARP
    arp.tpa = socket.inet_aton("192.168.0.38")

    eth_frame = struct.pack(
                        "!6s6sH",
                        eth.dst,
                        eth.src,
                        eth.etype)

    arp_frame = struct.pack(
                        "!HHBBH6s4s6s4s18s",
                        arp.htype,
                        arp.ptype,
                        arp.hsize,
                        arp.psize,
                        arp.op,
                        arp.shwa,
                        arp.spa,
                        arp.thwa,
                        arp.tpa,
                        arp.padd)

    print("creating ARP packet")
    arp_packet = eth_frame + arp_frame

    print("creating socket")
    s = socket.socket(socket.PF_PACKET,
                      socket.SOCK_RAW,
                      socket.htons(0x800))

    print("binding socket")
    s.bind((interface_name, 0))

    s.send(arp_packet)
